#if !defined(AFX_VBSTRING_H__AA037B93_4751_11D2_87BA_0080AD509054__INCLUDED_)
#define AFX_VBSTRING_H__AA037B93_4751_11D2_87BA_0080AD509054__INCLUDED_

/////////////////////////////////////////////////////////////////////////////
// CVbString is a wrapper class for the strange VB4 string type.
//
// The VB4 used ANSI (single-byte) BSTRs. These had to be
// allocated using the ::SysAllocStringByteLen() function.
//
// Written by Bjarke Viksoe (bjarke@viksoe.dk)
// Copyright (c) 2001 Bjarke Viksoe.
//
// This code may be used in compiled form in any way you desire. This
// file may be redistributed by any means PROVIDING it is 
// not sold for profit without the authors written consent, and 
// providing that this notice and the authors name is included. 
//
// This file is provided "as is" with no expressed or implied warranty.
// The author accepts no liability if it causes any damage to you or your
// computer whatsoever. It's free, so don't hassle me about it.
//
// Beware of bugs.
//

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


// We need ATL for quick UNICODE/ANSI conversions
// and other stuff...
#include <atlbase.h>



/////////////////////////////////////////////////////////////////////////////
// CVbString

// This is essentially a re-write of the
// ATL CComBSTR class

class CVbString
{
// Attributes
public:
	BSTR m_str;

// Implementation
public:
	CVbString()
	{
		m_str = NULL;
	}
	/*explicit*/ CVbString(int nSize)
	{
		m_str = ::SysAllocStringByteLen(NULL, nSize);
	}
	/*explicit*/ CVbString(int nSize, LPSTR sz)
	{
		m_str = ::SysAllocStringByteLen(sz, nSize);
	}
	/*explicit*/ CVbString(LPCOLESTR pSrc)
	{
		m_str = ::SysAllocString(pSrc);
	}
	/*explicit*/ CVbString(const CVbString& src)
	{
		m_str = src.Copy();
	}
	/*explicit*/ CVbString(REFGUID src)
	{
		LPOLESTR szGuid;
		StringFromCLSID(src, &szGuid);
		m_str = ::SysAllocString(szGuid);
		::CoTaskMemFree(szGuid);
	}
	CVbString& operator=(const CVbString& src)
	{
		if (m_str != src.m_str)
		{
			if (m_str) ::SysFreeString(m_str);
			m_str = src.Copy();
		}
		return *this;
	}

	CVbString& operator=(LPCOLESTR pSrc)
	{
		::SysFreeString(m_str);
		m_str = ::SysAllocString(pSrc);
		return *this;
	}

	~CVbString()
	{
		::SysFreeString(m_str);
	}
	unsigned int GetLength() const
	{
		return (m_str == NULL) ? 0 : SysStringByteLen(m_str);
	}
	operator BSTR() const
	{
		return m_str;
	}
	BSTR* operator&()
	{
		return &m_str;
	}
	BSTR Copy() const
	{
		return ::SysAllocStringByteLen((LPSTR)m_str, ::SysStringByteLen(m_str));
	}
	HRESULT CopyTo(BSTR* pbstr)
	{
		ASSERT(pbstr != NULL);
		if (pbstr == NULL) return E_POINTER;
		*pbstr = ::SysAllocStringByteLen((LPSTR)m_str, ::SysStringByteLen(m_str));
		if (*pbstr == NULL) return E_OUTOFMEMORY;
		return S_OK;
	}
	void Attach(BSTR src)
	{
		ASSERT(m_str == NULL);
		m_str = src;
	}
	BSTR Detach()
	{
		BSTR s = m_str;
		m_str = NULL;
		return s;
	}
	void Empty()
	{
		::SysFreeString(m_str);
		m_str = NULL;
	}
	bool operator!() const
	{
		return (m_str == NULL);
	}
	HRESULT Append(const CVbString& bstrSrc)
	{
		return Append(bstrSrc.m_str, ::SysStringByteLen(bstrSrc.m_str));
	}
	HRESULT Append(LPCOLESTR lpsz)
	{
		return Append(lpsz, ocslen(lpsz));
	}
	// a BSTR is just a LPCOLESTR so we need a special version to signify
	// that we are appending a BSTR
	HRESULT AppendBSTR(BSTR p)
	{
		return Append(p, SysStringByteLen(p));
	}
	HRESULT Append(LPCOLESTR lpsz, int nLen)
	{
		int n1 = GetLength();
		BSTR b;
		b = ::SysAllocStringByteLen(NULL, n1+nLen);
		if (b == NULL) return E_OUTOFMEMORY;
		memcpy(b, m_str, n1*sizeof(OLECHAR));
		memcpy(b+n1, lpsz, nLen*sizeof(OLECHAR));
		b[n1+nLen] = NULL;
		::SysFreeString(m_str);
		m_str = b;
		return S_OK;
	}
	HRESULT ToLower()
	{
		USES_CONVERSION;
		if (m_str != NULL) {
			LPTSTR psz = ::CharLower(OLE2T(m_str));
			if (psz == NULL) return E_OUTOFMEMORY;
			BSTR b = T2BSTR(psz);
			if (psz == NULL) return E_OUTOFMEMORY;
			::SysFreeString(m_str);
			m_str = b;
		}
		return S_OK;
	}
	HRESULT ToUpper()
	{
		USES_CONVERSION;
		if (m_str != NULL) {
			LPTSTR psz = ::CharUpper(OLE2T(m_str));
			if (psz == NULL) return E_OUTOFMEMORY;
			BSTR b = T2BSTR(psz);
			if (psz == NULL) return E_OUTOFMEMORY;
			::SysFreeString(m_str);
			m_str = b;
		}
		return S_OK;
	}
	bool LoadString(HINSTANCE hInst, UINT nID)
	{
		USES_CONVERSION;
		TCHAR sz[512];
		UINT nLen = ::LoadString(hInst, nID, sz, 512);
		ASSERT(nLen < 511);
		::SysFreeString(m_str);
		m_str = (nLen != 0) ? ::SysAllocString(T2OLE(sz)) : NULL;
		return (nLen != 0);
	}
	bool LoadString(UINT nID)
	{
		return LoadString(_pModule->m_hInstResource, nID);
	}

	CVbString& operator+=(const CVbString& bstrSrc)
	{
		AppendBSTR(bstrSrc.m_str);
		return *this;
	}
	bool operator<(BSTR bstrSrc) const
	{
		if (bstrSrc == NULL && m_str == NULL) return false;
		if (bstrSrc != NULL && m_str != NULL) return wcscmp(m_str, bstrSrc) < 0;
		return m_str == NULL;
	}
	bool operator==(BSTR bstrSrc) const
	{
		if (bstrSrc == NULL && m_str == NULL) return true;
		if (bstrSrc != NULL && m_str != NULL) return wcscmp(m_str, bstrSrc) == 0;
		return false;
	}
	bool operator<(LPCSTR pszSrc) const
	{
		if (pszSrc == NULL && m_str == NULL)
			return false;
		USES_CONVERSION;
		if (pszSrc != NULL && m_str != NULL)
			return wcscmp(m_str, A2W(pszSrc)) < 0;
		return m_str == NULL;
	}
	bool operator==(LPCSTR pszSrc) const
	{
		if (pszSrc == NULL && m_str == NULL)
			return true;
		USES_CONVERSION;
		if (pszSrc != NULL && m_str != NULL)
			return wcscmp(m_str, A2W(pszSrc)) == 0;
		return false;
	}
#ifndef OLE2ANSI
	CVbString(LPCSTR pSrc)
	{
		m_str = A2WBSTR(pSrc);
	}

	CVbString(int nSize, LPCSTR sz)
	{
		m_str = A2WBSTR(sz, nSize);
	}

	void Append(LPCSTR lpsz)
	{
		USES_CONVERSION;
		LPCOLESTR lpo = A2COLE(lpsz);
		Append(lpo, ocslen(lpo));
	}

	CVbString& operator=(LPCSTR pSrc)
	{
		::SysFreeString(m_str);
		m_str = A2WBSTR(pSrc);
		return *this;
	}
#endif
	HRESULT WriteToStream(IStream* pStream)
	{
		ASSERT(pStream != NULL);
		ULONG cb;
		ULONG cbStrLen = m_str ? ::SysStringByteLen(m_str)+sizeof(OLECHAR) : 0;
		HRESULT hr = pStream->Write((void*) &cbStrLen, sizeof(cbStrLen), &cb);
		if (FAILED(hr))
			return hr;
		return cbStrLen ? pStream->Write((void*) m_str, cbStrLen, &cb) : S_OK;
	}
	HRESULT ReadFromStream(IStream* pStream)
	{
		ASSERT(pStream != NULL);
		ASSERT(m_str == NULL); // should be empty
		ULONG cbStrLen = 0;
		HRESULT hr = pStream->Read((void*) &cbStrLen, sizeof(cbStrLen), NULL);
		if ((hr == S_OK) && (cbStrLen != 0))
		{
			//subtract size for terminating NULL which we wrote out
			//since SysAllocStringByteLen overallocates for the NULL
			m_str = ::SysAllocStringByteLen(NULL, cbStrLen-sizeof(OLECHAR));
			if (m_str == NULL)
				hr = E_OUTOFMEMORY;
			else
				hr = pStream->Read((void*) m_str, cbStrLen, NULL);
		}
		if (hr == S_FALSE)
			hr = E_FAIL;
		return hr;
	}
};

#endif // !defined(AFX_VBSTRING_H__AA037B93_4751_11D2_87BA_0080AD509054__INCLUDED_)
